#! /usr/bin/python2
'''
This file contains Python front end for collecting WCC data
1. Enable WCC data collect backend .
2. Disable WCC data collect  backend.

wcc_data_collect.py --enable                       # enable and start wcc data collection
wcc_data_collect.py --disable                      # disable and stop wcc data collection
'''

import argparse
import subprocess
import socket
import time
import os
import sys
import signal
import pickle
import cumulus.platforms

######################################################
#    defines
######################################################
WJH_BACKEND_SCRIPT = 'wcc_data_collect_backend.py'
WJH_DIR_SLASH = '/'
WJH_MONITOR_SERVER_PORT = 57000
WJH_TOOL_BUF_SIZE = 1024
WJH_SOCKET_TIMEOUT = 5
WJH_SERVER_UPTIME_WAIT_PERIOD = 2

# API to send/receive message to/from WJH example backend


def wcc_client_send(buf):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(WJH_SOCKET_TIMEOUT)
    server_address = ('localhost', WJH_MONITOR_SERVER_PORT)
    data = None
    try:
        sock.connect(server_address)
        sock.sendall(buf)
        data = sock.recv(WJH_TOOL_BUF_SIZE)
    except Exception:
        print("use --enable to enable the WCC data collection  backend.")
    finally:
        sock.close()
        return data


def wcc_action_enable(cmd):
    path = os.path.dirname(sys.argv[0])
    subprocess.Popen(["python", path + WJH_DIR_SLASH + WJH_BACKEND_SCRIPT])
    time.sleep(WJH_SERVER_UPTIME_WAIT_PERIOD)
    data = wcc_client_send(cmd)
    if data:
        print('%s' % data)
    else:
        print("Failed to enable the WCC backend - please try again")


def wcc_action_disable(cmd):
    data = wcc_client_send(cmd)
    if data:
        print '%s' % data
    else:
        print "Failed to disable the WCC backend - please check status and try again"


def handler(signum, frame):
    print 'Ctrl+C pressed, exit...'
    sys.exit(0)


platform_object = cumulus.platforms.probe()
if platform_object.switch.chip.sw_base != 'mlx':
    print "This script is currently supported on Mellanox platforms only\n"
    sys.exit(-1)

parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                 description='''WCC data collect front end does the following with respective options
1. Enable WCC data collect backend .
2. Disable WCC data collect  backend.

wcc_data_collect.py --enable                       # enable and start wcc data collection
wcc_data_collect.py --disable                      # disable and stop wcc data collection
''')

parser.add_argument('--disable', action='store_true', help='disables the WCC data collect backend')
parser.add_argument('--enable', action='store_true', help='enables the WCC data collect backend')

cmd = pickle.dumps(sys.argv[1:])
args = parser.parse_args()

if args.enable:
    wcc_action_enable(cmd)
elif args.disable:
    wcc_action_disable(cmd)
else:
    print("usage: wjh_example.py [-h|--help] [--enable] [--disable]")
